#include "logger.h"
#define INFO(src) BOOST_LOG_STREAM_WITH_PARAMS(src, (::boost::log::keywords::severity = logINFO))

BOOST_LOG_ATTRIBUTE_KEYWORD(severity, "Severity", LogLevelEnum)

// The operator puts a human-friendly representation of the severity level to the stream
std::ostream& operator<< (std::ostream& strm, LogLevelEnum level)
{
    static const char* strings[] =
    {
        "DEBUG",
        "INFO",
        "WARNING",
        "ERROR"
    };

    if (static_cast<std::size_t>(level) < sizeof(strings) / sizeof(*strings))
        strm << strings[level];
    else
        strm << static_cast<int>(level);

    return strm;
}

/*
 * If define the below template, then it is used in preference to the one
 * above for the application of operator<< onto the stream in the line
 * INFO(src) << ... << logINFO;
 * But it is not used to print the severity level by the severity logger.
 */
// template <typename T>
// boost::log::basic_record_ostream<T>& operator<< (boost::log::basic_record_ostream<T>& strm, LogLevelEnum)
// { return strm << "alternate"; }

typedef boost::log::sinks::synchronous_sink< boost::log::sinks::text_ostream_backend > console_sink;
boost::shared_ptr<console_sink> addConsoleLog()
{
    auto consoleformat = boost::log::expressions::stream
        << " [" << severity
        << "] " << ':' << boost::log::expressions::message;

    auto logConsoleSink = boost::make_shared<console_sink>();
    boost::shared_ptr<std::ostream> stream{ &std::cerr, boost::null_deleter{} };
    logConsoleSink->locked_backend()->add_stream(stream);
    logConsoleSink->set_formatter(consoleformat);
    boost::log::core::get()->add_sink(logConsoleSink);
    return logConsoleSink;
}

void LogInit()
{
    boost::log::add_common_attributes();

    boost::log::sources::severity_logger_mt<LogLevelEnum> src(
            boost::log::keywords::severity = logINFO);

    addConsoleLog();
    INFO(src) << "Logging to Console " << logINFO;
    // Without macros just so to help myself see things a little more
    // clearly.
    boost::log::record rec = src.open_record((::boost::log::keywords::severity = logINFO));
    if (rec) {
        boost::log::record_ostream strm(rec);
        strm << logINFO;
        strm.flush();
        src.push_record(boost::move(rec));
    }
}
// namespace core


boost::log::sources::severity_logger_mt<LogLevelEnum> slg_{};

void Init()
{
    std::cout << logINFO << std::endl;	
    LogInit();
}

void PrintLogLevel(LogLevelEnum level)
{
    std::cout << level << std::endl;
}
